import csv
import math
import random

import my_code_hw02
import numpy as np
import startinpy

NUMBER_RANDOM_TO_TEST = 10


random.seed(10)

with open("./samples.xyz") as csvfile:
    global dt
    dt = startinpy.DT()
    dt.snap_tolerance = 0.01
    r = csv.reader(csvfile, delimiter=" ")
    header = next(r)
    for line in r:
        p = list(map(float, line))  # -- convert each str to a float
        dt.insert_one_pt([p[0], p[1], p[2]])


class TestLinearTIN:
    def test_random_in_convexhull(self):
        bbox = dt.get_bbox()
        locs = []
        while True:
            x = random.uniform(bbox[0], bbox[2])
            y = random.uniform(bbox[1], bbox[3])
            if dt.is_inside_convex_hull([x, y]):
                locs.append([x, y])
            if len(locs) == NUMBER_RANDOM_TO_TEST:
                break
        tvs = dt.interpolate({"method": "TIN"}, locs)
        ivs = []
        for p in locs:
            re = my_code_hw02.interpolate_linear(dt, p[0], p[1])
            if isinstance(re, np.ndarray):
                ivs.append(re[0])
            else:
                ivs.append(re)
        assert np.all(np.isclose(tvs, ivs, atol=0.001))

    def test_outside_ch(self):
        bbox = dt.get_bbox()
        locs = []
        while True:
            x = random.uniform(bbox[0], bbox[2])
            y = random.uniform(bbox[1], bbox[3])
            if not dt.is_inside_convex_hull([x, y]):
                locs.append([x, y])
            if len(locs) == NUMBER_RANDOM_TO_TEST:
                break
        ivs = []
        for p in locs:
            re = my_code_hw02.interpolate_linear(dt, p[0], p[1])
            if isinstance(re, np.ndarray):
                ivs.append(re[0])
            else:
                ivs.append(re)
        assert np.all(np.isnan(ivs))

    def test_on_vertex(self):
        iv = my_code_hw02.interpolate_linear(dt, 215.242, 165.741)
        if isinstance(iv, np.ndarray):
            iv = iv[0]
        if isinstance(iv, list):
            iv = iv[0]
        assert math.isclose(iv, 62.273, abs_tol=0.001)

    def test_on_edge(self):
        dt = startinpy.DT()
        dt.insert_one_pt([0.0, 0.0, 1.0])
        dt.insert_one_pt([1.0, 0.0, 2.0])
        dt.insert_one_pt([1.1, 1.1, 3.0])
        dt.insert_one_pt([0.0, 1.0, 4.0])
        # iv = dt.interpolate({"method": "TIN"}, [[0.5, 0.5]])
        iv = my_code_hw02.interpolate_linear(dt, 0.5, 0.5)
        if isinstance(iv, np.ndarray):
            iv = iv[0]
        if isinstance(iv, list):
            iv = iv[0]
        assert math.isclose(3.0, iv, abs_tol=0.01)


class TestLaplace:
    def test_random_in_convexhull(self):
        bbox = dt.get_bbox()
        locs = []
        while True:
            x = random.uniform(bbox[0], bbox[2])
            y = random.uniform(bbox[1], bbox[3])
            if dt.is_inside_convex_hull([x, y]):
                locs.append([x, y])
            if len(locs) == NUMBER_RANDOM_TO_TEST:
                break
        tvs = dt.interpolate({"method": "Laplace"}, locs)
        ivs = []
        for p in locs:
            re = my_code_hw02.interpolate_laplace(dt, p[0], p[1])
            if isinstance(re, np.ndarray):
                ivs.append(re[0])
            else:
                ivs.append(re)
        assert np.all(np.isclose(tvs, ivs, atol=0.001))

    def test_outside_ch(self):
        bbox = dt.get_bbox()
        locs = []
        while True:
            x = random.uniform(bbox[0], bbox[2])
            y = random.uniform(bbox[1], bbox[3])
            if dt.is_inside_convex_hull([x, y]) == False:
                locs.append([x, y])
            if len(locs) == NUMBER_RANDOM_TO_TEST:
                break
        ivs = []
        for p in locs:
            re = my_code_hw02.interpolate_laplace(dt, p[0], p[1])
            if isinstance(re, np.ndarray):
                ivs.append(re[0])
            else:
                ivs.append(re)
        assert np.all(np.isnan(ivs))

    def test_on_vertex_no_interpol(self):
        iv = my_code_hw02.interpolate_laplace(dt, 215.242, 165.741)
        if isinstance(iv, np.ndarray):
            iv = iv[0]
        if isinstance(iv, list):
            iv = iv[0]
        assert math.isclose(iv, 62.273, abs_tol=0.001)

    def test_on_vertex_removed(self):
        dt = startinpy.DT()
        dt.insert_one_pt([0.0, 0.0, 1.0])
        dt.insert_one_pt([1.0, 0.0, 2.0])
        dt.insert_one_pt([1.1, 1.1, 3.0])
        dt.insert_one_pt([0.0, 1.0, 4.0])
        dt.insert_one_pt([0.5, 0.5, 10.0])
        bn = dt.number_of_vertices()
        my_code_hw02.interpolate_laplace(dt, 0.5, 0.5)
        an = dt.number_of_vertices()
        assert bn == an
